{% extends "site_base.html" %}
{% load i18n %}
{% load bootstrap_tags %}

{% block body %}

  <h2>{% trans "Invite User" %}</h2>
  
  <form action="" class="form-horizontal" method="post">
    
    {% csrf_token %}
      {{ form|as_bootstrap }}
    <div class="form-actions">
      <input type="submit" value="{% trans "Invite User" %}" class="btn btn-primary"/>
    </div>
    
  </form>
  
{% endblock %}

